use serde::Serialize;

use super::angle_link::WasmAngleLink;
use super::babel_call::WasmBabelCall;
use super::bold::WasmBold;
use super::center_block::WasmCenterBlock;
use super::citation::WasmCitation;
use super::citation_reference::WasmCitationReference;
use super::clock::WasmClock;
use super::code::WasmCode;
use super::comment::WasmComment;
use super::comment_block::WasmCommentBlock;
use super::diary_sexp::WasmDiarySexp;
use super::document::WasmDocument;
use super::drawer::WasmDrawer;
use super::dynamic_block::WasmDynamicBlock;
use super::entity::WasmEntity;
use super::example_block::WasmExampleBlock;
use super::export_block::WasmExportBlock;
use super::export_snippet::WasmExportSnippet;
use super::fixed_width_area::WasmFixedWidthArea;
use super::footnote_definition::WasmFootnoteDefinition;
use super::footnote_reference::WasmFootnoteReference;
use super::headline::WasmHeadline;
use super::horizontal_rule::WasmHorizontalRule;
use super::inline_babel_call::WasmInlineBabelCall;
use super::inline_source_block::WasmInlineSourceBlock;
use super::italic::WasmItalic;
use super::keyword::WasmKeyword;
use super::latex_environment::WasmLatexEnvironment;
use super::latex_fragment::WasmLatexFragment;
use super::line_break::WasmLineBreak;
use super::node_property::WasmNodeProperty;
use super::org_macro::WasmOrgMacro;
use super::paragraph::WasmParagraph;
use super::plain_link::WasmPlainLink;
use super::plain_list::WasmPlainList;
use super::plain_list_item::WasmPlainListItem;
use super::plain_text::WasmPlainText;
use super::planning::WasmPlanning;
use super::property_drawer::WasmPropertyDrawer;
use super::quote_block::WasmQuoteBlock;
use super::radio_link::WasmRadioLink;
use super::radio_target::WasmRadioTarget;
use super::regular_link::WasmRegularLink;
use super::section::WasmSection;
use super::special_block::WasmSpecialBlock;
use super::src_block::WasmSrcBlock;
use super::statistics_cookie::WasmStatisticsCookie;
use super::strike_through::WasmStrikeThrough;
use super::subscript::WasmSubscript;
use super::superscript::WasmSuperscript;
use super::table::WasmTable;
use super::table_cell::WasmTableCell;
use super::table_row::WasmTableRow;
use super::target::WasmTarget;
use super::timestamp::WasmTimestamp;
use super::underline::WasmUnderline;
use super::verbatim::WasmVerbatim;
use super::verse_block::WasmVerseBlock;

#[derive(Debug, Serialize)]
#[serde(untagged)]
pub enum WasmAstNode<'s> {
    // Document Nodes
    Document(WasmDocument<'s>),
    Headline(WasmHeadline<'s>),
    Section(WasmSection<'s>),
    // Elements
    Paragraph(WasmParagraph<'s>),
    PlainList(WasmPlainList<'s>),
    PlainListItem(WasmPlainListItem<'s>),
    CenterBlock(WasmCenterBlock<'s>),
    QuoteBlock(WasmQuoteBlock<'s>),
    SpecialBlock(WasmSpecialBlock<'s>),
    DynamicBlock(WasmDynamicBlock<'s>),
    FootnoteDefinition(WasmFootnoteDefinition<'s>),
    Comment(WasmComment<'s>),
    Drawer(WasmDrawer<'s>),
    PropertyDrawer(WasmPropertyDrawer<'s>),
    NodeProperty(WasmNodeProperty<'s>),
    Table(WasmTable<'s>),
    TableRow(WasmTableRow<'s>),
    VerseBlock(WasmVerseBlock<'s>),
    CommentBlock(WasmCommentBlock<'s>),
    ExampleBlock(WasmExampleBlock<'s>),
    ExportBlock(WasmExportBlock<'s>),
    SrcBlock(WasmSrcBlock<'s>),
    Clock(WasmClock<'s>),
    DiarySexp(WasmDiarySexp<'s>),
    Planning(WasmPlanning<'s>),
    FixedWidthArea(WasmFixedWidthArea<'s>),
    HorizontalRule(WasmHorizontalRule<'s>),
    Keyword(WasmKeyword<'s>),
    BabelCall(WasmBabelCall<'s>),
    LatexEnvironment(WasmLatexEnvironment<'s>),
    // Objects
    Bold(WasmBold<'s>),
    Italic(WasmItalic<'s>),
    Underline(WasmUnderline<'s>),
    StrikeThrough(WasmStrikeThrough<'s>),
    Code(WasmCode<'s>),
    Verbatim(WasmVerbatim<'s>),
    PlainText(WasmPlainText<'s>),
    RegularLink(WasmRegularLink<'s>),
    RadioLink(WasmRadioLink<'s>),
    RadioTarget(WasmRadioTarget<'s>),
    PlainLink(WasmPlainLink<'s>),
    AngleLink(WasmAngleLink<'s>),
    OrgMacro(WasmOrgMacro<'s>),
    Entity(WasmEntity<'s>),
    LatexFragment(WasmLatexFragment<'s>),
    ExportSnippet(WasmExportSnippet<'s>),
    FootnoteReference(WasmFootnoteReference<'s>),
    Citation(WasmCitation<'s>),
    CitationReference(WasmCitationReference<'s>),
    InlineBabelCall(WasmInlineBabelCall<'s>),
    InlineSourceBlock(WasmInlineSourceBlock<'s>),
    LineBreak(WasmLineBreak<'s>),
    Target(WasmTarget<'s>),
    StatisticsCookie(WasmStatisticsCookie<'s>),
    Subscript(WasmSubscript<'s>),
    Superscript(WasmSuperscript<'s>),
    TableCell(WasmTableCell<'s>),
    Timestamp(WasmTimestamp<'s>),
}

impl<'s> WasmAstNode<'s> {}
